@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,13 +25,16 @@
 #include "gpu/ce/kernel_ce.h"
 #include "gpu/ce/kernel_ce_private.h"
 #include "gpu/eng_desc.h"
-#include "gpu/subdevice/subdevice.h"
+#include "gpu/mem_mgr/ce_utils.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "kernel/gpu/intr/intr_service.h"
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
 #include "kernel/gpu/nvlink/common_nvlink.h"
+#include "vgpu/sdk-structures.h"
 #include "nvRmReg.h"
 
+#include "gpu/conf_compute/ccsl.h"
+
 NV_STATUS kceConstructEngine_IMPL(OBJGPU *pGpu, KernelCE *pKCe, ENGDESCRIPTOR engDesc)
 {
     NV_ASSERT_OR_RETURN(!RMCFG_FEATURE_PLATFORM_GSP, NV_ERR_NOT_SUPPORTED);
@@ -62,7 +65,7 @@ NV_STATUS kceConstructEngine_IMPL(OBJGPU *pGpu, KernelCE *pKCe, ENGDESCRIPTOR en
     }
 
     // OBJCE::isPresent would compute this first
-    if (IS_GSP_CLIENT(pGpu))
+    if (IS_GSP_CLIENT(pGpu) || IS_VIRTUAL(pGpu))
     {
         pGpu->numCEs++;
     }
@@ -90,6 +93,195 @@ NvBool kceIsNewMissingEngineRemovalSequenceEnabled_IMPL(OBJGPU *pGpu, KernelCE *
     return NV_TRUE;
 }
 
+#define CE_FIPS_SELF_TEST_DATA_SIZE 16
+#define CE_FIPS_SELF_TEST_AUTH_TAG_SIZE 16
+#define CE_FIPS_SELF_TEST_IV_SIZE 12
+
+NV_STATUS
+kceRunFipsSelfTest
+(
+    OBJGPU   *pGpu,
+    void     *pArg
+)
+{
+    KernelCE          *pKCe              = pArg;
+    MemoryManager     *pMemoryManager    = GPU_GET_MEMORY_MANAGER(pGpu);
+    KernelMIGManager  *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
+    MEMORY_DESCRIPTOR *pSrcMemDesc       = NULL;
+    MEMORY_DESCRIPTOR *pDstMemDesc       = NULL;
+    MEMORY_DESCRIPTOR *pAuthMemDesc      = NULL;
+    MEMORY_DESCRIPTOR *pIvMemDesc        = NULL;
+    CeUtils           *pCeUtils          = NULL;
+    pCcslContext       pCcslCtx          = NULL;
+    NV_STATUS          status;
+    NV0050_ALLOCATION_PARAMETERS ceUtilsParams = {0};
+    CEUTILS_MEMCOPY_PARAMS params = {0};
+
+    NvU8 ceTestPlaintext[CE_FIPS_SELF_TEST_DATA_SIZE] = {
+        0x2d, 0x71, 0xbc, 0xfa, 0x91, 0x4e, 0x4a, 0xc0,
+        0x45, 0xb2, 0xaa, 0x60, 0x95, 0x5f, 0xad, 0x24
+    };
+    NvU8 decryptedData[CE_FIPS_SELF_TEST_DATA_SIZE] = { 0 };
+    NvU8 encryptedData[CE_FIPS_SELF_TEST_DATA_SIZE] = { 0 };
+    NvU8 dataAuth[CE_FIPS_SELF_TEST_AUTH_TAG_SIZE] = { 0 };
+
+    NV_ASSERT_OR_RETURN(gpuIsCCFeatureEnabled(pGpu), NV_ERR_NOT_SUPPORTED);
+
+    if (!gpuCheckEngineTable(pGpu, RM_ENGINE_TYPE_COPY(pKCe->publicID)) ||
+        ceIsCeGrce(pGpu, RM_ENGINE_TYPE_COPY(pKCe->publicID)))
+    {
+        // CE doesn't support encryption
+        return NV_OK;
+    }
+
+    if (kmigmgrIsMIGSupported(pGpu, pKernelMIGManager) &&
+        kmigmgrGetStaticInfo(pGpu, pKernelMIGManager) == NULL)
+    {
+        // Wait for KernelMigManager, as it might remap CEs
+        return NV_WARN_MORE_PROCESSING_REQUIRED;
+    }
+
+    NV_PRINTF(LEVEL_INFO, "Running FIPS test for CE%u\n", pKCe->publicID);
+
+    ceUtilsParams.flags |= DRF_DEF(0050_CEUTILS, _FLAGS, _FORCE_CE_ID, _TRUE);
+    ceUtilsParams.flags |= DRF_DEF(0050_CEUTILS, _FLAGS, _CC_SECURE, _TRUE);
+    ceUtilsParams.forceCeId = pKCe->publicID;
+
+    NV_ASSERT_OK_OR_GOTO(status,
+        objCreate(&pCeUtils, pMemoryManager, CeUtils, ENG_GET_GPU(pMemoryManager), NULL, &ceUtilsParams), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status,
+        ccslContextInitViaChannel_HAL(&pCcslCtx, pCeUtils->pChannel->hClient,
+                                      pCeUtils->pChannel->channelId),
+        failed);
+
+    NV_ASSERT_OK_OR_GOTO(status, memdescCreate(&pSrcMemDesc, pGpu, sizeof ceTestPlaintext, 0, NV_TRUE, ADDR_FBMEM,
+                         NV_MEMORY_UNCACHED, MEMDESC_ALLOC_FLAGS_PROTECTED), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(pSrcMemDesc), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status, memdescCreate(&pDstMemDesc, pGpu, sizeof encryptedData, 0, NV_TRUE, ADDR_SYSMEM,
+                         NV_MEMORY_UNCACHED, MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(pDstMemDesc), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status, memdescCreate(&pAuthMemDesc, pGpu, sizeof dataAuth, 0, NV_TRUE, ADDR_SYSMEM,
+                         NV_MEMORY_UNCACHED, MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(pAuthMemDesc), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status, memdescCreate(&pIvMemDesc, pGpu, CE_FIPS_SELF_TEST_IV_SIZE, 0, NV_TRUE, ADDR_SYSMEM,
+                         NV_MEMORY_UNCACHED, MEMDESC_FLAGS_ALLOC_IN_UNPROTECTED_MEMORY), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memdescAlloc(pIvMemDesc), failed);
+
+    TRANSFER_SURFACE srcSurface  = { .pMemDesc = pSrcMemDesc,  .offset = 0 };
+    TRANSFER_SURFACE dstSurface  = { .pMemDesc = pDstMemDesc,  .offset = 0 };
+    TRANSFER_SURFACE authSurface = { .pMemDesc = pAuthMemDesc, .offset = 0 };
+
+    // Write data to allocations, encrypt using CE, and read back the results
+    NV_ASSERT_OK_OR_GOTO(status, memmgrMemDescMemSet(pMemoryManager, pDstMemDesc, 0, 0), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memmgrMemDescMemSet(pMemoryManager, pAuthMemDesc, 0, 0), failed);
+    NV_ASSERT_OK_OR_GOTO(status,
+        memmgrMemWrite(pMemoryManager, &srcSurface, ceTestPlaintext, sizeof ceTestPlaintext, TRANSFER_FLAGS_NONE),
+        failed);
+
+    params.bSecureCopy    = NV_TRUE;
+    params.authTagAddr    = memdescGetPhysAddr(pAuthMemDesc, AT_GPU, 0);
+    params.encryptIvAddr  = memdescGetPhysAddr(pIvMemDesc, AT_GPU, 0);
+    params.pDstMemDesc    = pDstMemDesc;
+    params.dstOffset      = 0;
+    params.pSrcMemDesc    = pSrcMemDesc;
+    params.srcOffset      = 0;
+    params.length         = sizeof ceTestPlaintext;
+    params.bEncrypt       = NV_TRUE;
+    NV_ASSERT_OK_OR_GOTO(status, ceutilsMemcopy(pCeUtils, &params), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status,
+        memmgrMemRead(pMemoryManager, &dstSurface, encryptedData, sizeof encryptedData, TRANSFER_FLAGS_NONE), failed);
+    NV_ASSERT_OK_OR_GOTO(status,
+        memmgrMemRead(pMemoryManager, &authSurface, dataAuth, sizeof dataAuth, TRANSFER_FLAGS_NONE), failed);
+
+    // Decrypt using CPU and validate
+    NV_ASSERT_OK_OR_GOTO(status,
+        ccslDecrypt_HAL(pCcslCtx, sizeof decryptedData, encryptedData, NULL, NULL, 0, decryptedData, dataAuth),
+        failed);
+
+    NV_ASSERT_TRUE_OR_GOTO(status, portMemCmp(decryptedData, ceTestPlaintext, sizeof ceTestPlaintext) == 0,
+        NV_ERR_INVALID_STATE, failed);
+
+    // Encrypt using CPU
+    NV_ASSERT_OK_OR_GOTO(status,
+        ccslEncrypt_HAL(pCcslCtx, sizeof ceTestPlaintext, ceTestPlaintext, NULL, 0, encryptedData, dataAuth), failed);
+
+    // Write data to allocations, decrypt using CE, read back, and validate
+    NV_ASSERT_OK_OR_GOTO(status,
+        memmgrMemWrite(pMemoryManager, &dstSurface, encryptedData, sizeof encryptedData, TRANSFER_FLAGS_NONE), failed);
+    NV_ASSERT_OK_OR_GOTO(status,
+        memmgrMemWrite(pMemoryManager, &authSurface, dataAuth, sizeof dataAuth, TRANSFER_FLAGS_NONE), failed);
+    NV_ASSERT_OK_OR_GOTO(status, memmgrMemDescMemSet(pMemoryManager, pSrcMemDesc, 0, 0), failed);
+
+    params.pDstMemDesc = pSrcMemDesc;
+    params.dstOffset   = 0;
+    params.pSrcMemDesc = pDstMemDesc;
+    params.srcOffset   = 0;
+    params.length      = sizeof ceTestPlaintext;
+    params.bEncrypt    = NV_FALSE;
+    NV_ASSERT_OK_OR_GOTO(status, ceutilsMemcopy(pCeUtils, &params), failed);
+
+    NV_ASSERT_OK_OR_GOTO(status,
+        memmgrMemRead(pMemoryManager, &srcSurface, decryptedData, sizeof decryptedData, TRANSFER_FLAGS_NONE), failed);
+
+    NV_ASSERT_TRUE_OR_GOTO(status, portMemCmp(decryptedData, ceTestPlaintext, sizeof ceTestPlaintext) == 0,
+        NV_ERR_INVALID_STATE, failed);
+
+failed:
+    ccslContextClear(pCcslCtx);
+    objDelete(pCeUtils);
+    memdescFree(pSrcMemDesc);
+    memdescDestroy(pSrcMemDesc);
+    memdescFree(pDstMemDesc);
+    memdescDestroy(pDstMemDesc);
+    memdescFree(pAuthMemDesc);
+    memdescDestroy(pAuthMemDesc);
+    memdescFree(pIvMemDesc);
+    memdescDestroy(pIvMemDesc);
+
+    NV_PRINTF(LEVEL_INFO, "Test finished with status 0x%x\n", status);
+
+    return status;
+}
+
+NV_STATUS
+kceStateInitLocked_IMPL
+(
+    OBJGPU   *pGpu,
+    KernelCE *pKCe
+)
+{
+    if (!gpuIsCCFeatureEnabled(pGpu) || !IS_SILICON(pGpu))
+    {
+        pKCe->bCcFipsSelfTestRequired = NV_FALSE;
+    }
+
+    if (pKCe->bCcFipsSelfTestRequired)
+    {
+        NV_ASSERT_OK_OR_RETURN(
+            kfifoAddSchedulingHandler(pGpu, GPU_GET_KERNEL_FIFO(pGpu), kceRunFipsSelfTest, pKCe, NULL, NULL));
+    }
+
+    return NV_OK;
+}
+
+void
+kceStateDestroy_IMPL
+(
+    OBJGPU   *pGpu,
+    KernelCE *pKCe
+)
+{
+    if (pKCe->bCcFipsSelfTestRequired)
+    {
+        kfifoRemoveSchedulingHandler(pGpu, GPU_GET_KERNEL_FIFO(pGpu), kceRunFipsSelfTest, pKCe, NULL, NULL);
+    }
+}
+
 static void printCaps(OBJGPU *pGpu, KernelCE *pKCe, RM_ENGINE_TYPE rmEngineType, const NvU8 *capsTbl)
 {
     NV_PRINTF(LEVEL_INFO, "LCE%d caps (engineType = %d (%d))\n", pKCe->publicID,
@@ -424,6 +616,17 @@ NV_STATUS kceGetFaultMethodBufferSize_IMPL(OBJGPU *pGpu, NvU32 *size)
     RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
     NV2080_CTRL_CE_GET_FAULT_METHOD_BUFFER_SIZE_PARAMS params = {0};
 
+    if (IS_VIRTUAL_WITH_SRIOV(pGpu))
+    {
+        VGPU_STATIC_INFO *pVSI = GPU_GET_STATIC_INFO(pGpu);
+
+        NV_ASSERT_OR_RETURN(pVSI, NV_ERR_INVALID_STATE);
+
+        *size = pVSI->ceFaultMethodBufferDepth;
+
+        return NV_OK;
+    }
+
     NV_ASSERT_OK_OR_RETURN(pRmApi->Control(pRmApi, pGpu->hInternalClient, pGpu->hInternalSubdevice,
             NV2080_CTRL_CMD_CE_GET_FAULT_METHOD_BUFFER_SIZE, &params, sizeof(params)));
 
